from typing import Any

from .ContainerIO import ContainerIO

class TarIO(ContainerIO):
    fh: Any
    def __init__(self, tarfile, file) -> None: ...
    def __enter__(self): ...
    def __exit__(self, *args) -> None: ...
    def close(self) -> None: ...
