from _typeshed import Incomplete
from typing import Any

class PhotoImage:
    tk: Any
    def __init__(self, image: Incomplete | None = ..., size: Incomplete | None = ..., **kw) -> None: ...
    def __del__(self) -> None: ...
    def width(self): ...
    def height(self): ...
    def paste(self, im, box: Incomplete | None = ...) -> None: ...

class BitmapImage:
    def __init__(self, image: Incomplete | None = ..., **kw) -> None: ...
    def __del__(self) -> None: ...
    def width(self): ...
    def height(self): ...

def getimage(photo): ...
