from _typeshed import Incomplete
from typing import Any

class Pen:
    color: Any
    width: Any
    def __init__(self, color, width: int = ..., opacity: int = ...) -> None: ...

class Brush:
    color: Any
    def __init__(self, color, opacity: int = ...) -> None: ...

class Font:
    color: Any
    font: Any
    def __init__(self, color, file, size: int = ...) -> None: ...

class Draw:
    draw: Any
    image: Any
    transform: Any
    def __init__(self, image, size: Incomplete | None = ..., color: Incomplete | None = ...) -> None: ...
    def flush(self): ...
    def render(self, op, xy, pen, brush: Incomplete | None = ...) -> None: ...
    def settransform(self, offset) -> None: ...
    def arc(self, xy, start, end, *options) -> None: ...
    def chord(self, xy, start, end, *options) -> None: ...
    def ellipse(self, xy, *options) -> None: ...
    def line(self, xy, *options) -> None: ...
    def pieslice(self, xy, start, end, *options) -> None: ...
    def polygon(self, xy, *options) -> None: ...
    def rectangle(self, xy, *options) -> None: ...
    def text(self, xy, text, font) -> None: ...
    def textsize(self, text, font): ...
