from typing import Any

EPSILON: float

def linear(middle, pos): ...
def curved(middle, pos): ...
def sine(middle, pos): ...
def sphere_increasing(middle, pos): ...
def sphere_decreasing(middle, pos): ...

SEGMENTS: Any

class GradientFile:
    gradient: Any
    def getpalette(self, entries: int = ...): ...

class GimpGradientFile(GradientFile):
    gradient: Any
    def __init__(self, fp) -> None: ...
